'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _events = require('events');

var _events2 = _interopRequireDefault(_events);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('mz/fs');

var _fs2 = _interopRequireDefault(_fs);

var _nodeCache = require('node-cache');

var _nodeCache2 = _interopRequireDefault(_nodeCache);

var _glob = require('glob');

var _glob2 = _interopRequireDefault(_glob);

var _sane = require('sane');

var _sane2 = _interopRequireDefault(_sane);

var _micromatch = require('micromatch');

var _micromatch2 = _interopRequireDefault(_micromatch);

var _log = require('./log');

var log = _interopRequireWildcard(_log);

var _javascript = require('./javascript');

var _css = require('./css');

var _css2 = _interopRequireDefault(_css);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

var Repository = function (_EventEmitter) {
  _inherits(Repository, _EventEmitter);

  function Repository(_ref) {
    var name = _ref.name,
        directory = _ref.directory,
        _ref$watch = _ref.watch,
        watch = _ref$watch === undefined ? true : _ref$watch;

    _classCallCheck(this, Repository);

    var _this = _possibleConstructorReturn(this, (Repository.__proto__ || Object.getPrototypeOf(Repository)).call(this));

    _this.broken = false;
    _this.name = name;
    _this.dir = _path2.default.normalize(directory);
    if (_this.dir.endsWith(_path2.default.sep)) _this.dir = _this.dir.slice(0, -1);
    _this.loadFiles();
    if (watch) _this.watch();
    return _this;
  }

  _createClass(Repository, [{
    key: 'loadFiles',
    value: function loadFiles() {
      var _this2 = this;

      try {
        _fs2.default.statSync(this.dir);
        this.files = [];
        this.cache = new _nodeCache2.default({ stdTTL: 600 });
        this.loadConfig();
        this.ready = new Promise(function (resolve) {
          var opts = { cwd: _this2.dir, ignore: 'node_modules/**', nodir: true };
          (0, _glob2.default)('**/*', opts, function (error, files) {
            _this2.files = files.map(function (filename) {
              return { filename: filename };
            });
            _this2.dedupeFiles();
            resolve();
          });
        });
      } catch (error) {
        this.broken = true;
        this.ready = Promise.resolve();
        if (error.code !== 'ENOENT') log.error(error);
      }
    }
  }, {
    key: 'watch',
    value: function watch() {
      var _this3 = this;

      this.watcher = (0, _sane2.default)(this.dir, { ignored: /node_modules/ }).on('error', function () {
        return log.error('error', _this3.name);
      }).on('add', function (path) {
        return _this3.add(path.replace(/\\/g, '/'));
      }).on('change', function (path) {
        return _this3.change(path.replace(/\\/g, '/'));
      }).on('delete', function (path) {
        return _this3.remove(path.replace(/\\/g, '/'));
      });
    }
  }, {
    key: 'close',
    value: function close() {
      if (this.watcher) this.watcher.close();
    }
  }, {
    key: 'toJSON',
    value: function toJSON() {

      if (this.broken) {
        return {
          name: this.name,
          path: this.dir,
          broken: true
        };
      }

      var assets = this.files.map(function (a) {
        return a.path;
      }).filter(function (p) {
        return !!p;
      }).sort(function (a, b) {
        if (a < b) return -1;else if (a > b) return 1;else return 0;
      });

      var files = this.files.sort(function (a, b) {
        if (a.filename < b.filename) return -1;else if (a.filename > b.filename) return 1;else return 0;
      });

      return {
        name: this.name,
        path: this.dir,
        assets: assets,
        files: files
      };
    }
  }, {
    key: 'findAsset',
    value: function findAsset(assetPath) {
      return this.files.find(function (f) {
        return f.path === assetPath;
      });
    }
  }, {
    key: 'findFilename',
    value: function findFilename(assetPath) {
      var asset = this.findAsset(assetPath);
      if (asset) return asset.filename;
    }
  }, {
    key: 'assetPath',
    value: function assetPath(filename) {
      var ext = _path2.default.extname(filename).slice(1);

      var assetPath = '';

      // can't have strange characters, including underscores
      if (/[^-/.a-z0-9]/i.test(filename)) return null;
      // .less files become .css
      else if ('less' == ext) assetPath = filename.slice(0, -4) + 'css';
        // .dev.css becomes .css
        else if (/\.dev\.(js|css)$/.test(filename)) assetPath = filename.replace(/\.dev\.(js|css)$/, '.$1');
          // normal extensions
          else if (['css', 'js', 'svg', 'html', 'ttf', 'woff', 'woff2'].includes(ext)) assetPath = filename;
            // other things are ignored
            else return null;

      // Windows backslash nightmare
      if (_path2.default.sep === '\\') assetPath = assetPath.replace(/\\/g, '/');

      return '/' + this.name + '/' + assetPath;
    }
  }, {
    key: 'loadConfig',
    value: function loadConfig() {

      this.ignoreMatchers = ['**/_*'];

      this.config = {
        script: [],
        ignore: []
      };

      try {
        var json = _fs2.default.readFileSync(this.dir + '/package.json');
        var config = JSON.parse(json).codex || {};

        var validStrArr = function validStrArr(prop) {
          var valid = !(prop in config) || config[prop] instanceof Array && config[prop].every(function (s) {
            return typeof s === 'string';
          });

          if (valid) return config[prop];else log.error('"' + prop + '" section of Codex configuration must be an array of strings');
        };

        this.config.script = validStrArr('script') || [];
        this.config.ignore = validStrArr('ignore') || [];
        if (validStrArr('only')) this.config.only = config.only;
      } catch (e) {
        if (e.code !== 'ENOENT') log.error(e);

        return;
      }

      this.ignoreMatchers = this.config.ignore.reduce(function (ignore, i) {
        ignore.push(i);
        // recursively ignore by default
        ignore.push(i.endsWith('/') ? i + '**/*' : i + '/**/*');
        return ignore;
      }, this.ignoreMatchers);

      this.dedupeFiles();
    }
  }, {
    key: 'code',
    value: function () {
      var _ref2 = _asyncToGenerator(regeneratorRuntime.mark(function _callee(assetPath) {
        var _ref3 = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {},
            _ref3$useModules = _ref3.useModules,
            useModules = _ref3$useModules === undefined ? false : _ref3$useModules;

        var asset, cached, _ref4, deps, _code, script, _code2, _code3;

        return regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                asset = this.files.find(function (f) {
                  return f.path === assetPath;
                });

                if (asset) {
                  _context.next = 3;
                  break;
                }

                throw new Error('Not found: ' + assetPath);

              case 3:
                cached = this.cache.get(assetPath);

                if (!(cached && !useModules)) {
                  _context.next = 6;
                  break;
                }

                return _context.abrupt('return', cached);

              case 6:
                _context.prev = 6;

                if (!/\.css$/.test(assetPath)) {
                  _context.next = 18;
                  break;
                }

                _context.next = 10;
                return (0, _css2.default)({
                  repositoryName: this.name,
                  assetPath: assetPath,
                  filename: asset.filename,
                  directory: this.dir
                });

              case 10:
                _ref4 = _context.sent;
                deps = _ref4.deps;
                _code = _ref4.code;


                this.cache.set(assetPath, _code);
                asset.deps = deps;
                return _context.abrupt('return', _code);

              case 18:
                if (!/\.js$/.test(assetPath)) {
                  _context.next = 37;
                  break;
                }

                script = this.config.script.includes(asset.filename);

                if (!useModules) {
                  _context.next = 24;
                  break;
                }

                return _context.abrupt('return', _fs2.default.readFile(_path2.default.join(this.dir, asset.filename), 'utf-8'));

              case 24:
                if (!script) {
                  _context.next = 32;
                  break;
                }

                _context.next = 27;
                return (0, _javascript.transpileScript)({
                  assetPath: assetPath,
                  filename: asset.filename,
                  directory: this.dir
                });

              case 27:
                _code2 = _context.sent;

                this.cache.set(assetPath, _code2);
                return _context.abrupt('return', _code2);

              case 32:
                _context.next = 34;
                return (0, _javascript.transpileModule)({
                  assetPath: assetPath,
                  filename: asset.filename,
                  directory: this.dir
                });

              case 34:
                _code3 = _context.sent;

                this.cache.set(assetPath, _code3);
                return _context.abrupt('return', _code3);

              case 37:
                _context.next = 43;
                break;

              case 39:
                _context.prev = 39;
                _context.t0 = _context['catch'](6);


                this.emit('message', {
                  event: 'error',
                  repositoryName: this.name,
                  filename: asset.filename,
                  paths: this.findAffectedAssetPaths(asset.filename),
                  error: _context.t0
                });

                throw _context.t0;

              case 43:
              case 'end':
                return _context.stop();
            }
          }
        }, _callee, this, [[6, 39]]);
      }));

      function code(_x) {
        return _ref2.apply(this, arguments);
      }

      return code;
    }()
  }, {
    key: 'shouldIgnore',
    value: function shouldIgnore(filename) {
      if (!/\.(js|css|less|svg|html|ttf|woff|woff2)$/.test(filename)) return true;else if (this.config.only) return !this.config.only.includes(filename);else return _micromatch2.default.any(filename, this.ignoreMatchers);
    }
  }, {
    key: 'findAffectedAssetPaths',
    value: function findAffectedAssetPaths(filename) {
      return this.files.filter(function (f) {
        return f.deps && f.deps.includes(filename);
      }).map(function (f) {
        return f.path;
      }).concat([this.assetPath(filename)]);
    }
  }, {
    key: 'change',
    value: function change(filename) {
      var _this4 = this;

      log.cyan('change ' + this.name + ':  ' + filename);

      if (filename === 'package.json') {
        this.loadConfig();
        this.emit('message', {
          repositoryName: this.name,
          event: 'change',
          filename: filename
        });
        return;
      }

      var file = this.files.find(function (f) {
        return f.filename === filename;
      });

      if (file.path) {
        var paths = this.findAffectedAssetPaths(filename);
        paths.forEach(function (p) {
          return _this4.cache.del(p);
        });
        this.emit('message', {
          repositoryName: this.name,
          event: 'change',
          filename: filename,
          paths: paths
        });
      }
    }
  }, {
    key: 'add',
    value: function add(filename) {
      var silent = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;

      log.cyan('add ' + this.name + ':  ' + filename);

      if (this.shouldIgnore(filename)) this.files.push({ filename: filename });else this.files.push({ filename: filename, path: this.assetPath(filename) });

      if (filename === 'package.json') {
        this.loadConfig();
        this.emit('message', {
          repositoryName: this.name,
          event: 'add',
          filename: filename
        });
        return;
      } else {
        this.dedupeFiles();
      }

      if (!this.shouldIgnore(filename)) {
        this.emit('message', {
          repositoryName: this.name,
          event: 'add',
          filename: filename,
          paths: [this.assetPath(filename)]
        });
      }
    }
  }, {
    key: 'remove',
    value: function remove(filename) {
      log.cyan('remove ' + this.name + ':  ' + filename);

      this.files = this.files.filter(function (f) {
        return f.filename !== filename;
      });

      this.dedupeFiles();

      if (!this.shouldIgnore(filename)) {
        this.emit('message', {
          repositoryName: this.name,
          event: 'remove',
          filename: filename,
          paths: [this.assetPath(filename)]
        });
      }
    }
  }, {
    key: 'dedupeFiles',
    value: function dedupeFiles() {
      var _this5 = this;

      // add asset paths to everything
      this.files = this.files.map(function (f) {
        if (_this5.shouldIgnore(f.filename)) return { filename: f.filename };else return { filename: f.filename, path: _this5.assetPath(f.filename) };
      });

      // remove asset paths from the ones that have duplicates
      this.files.forEach(function (f) {
        if (!f.path) return;

        _this5.files.filter(function (dup) {
          return dup.path === f.path && dup.filename != f.filename;
        }).forEach(function (dup) {

          var weaker = f.filename.endsWith('.less') || dup.filename.endsWith('.dev.js') || dup.filename.endsWith('.dev.css');

          if (weaker) delete f.path;
        });
      });
    }
  }]);

  return Repository;
}(_events2.default);

exports.default = Repository;