'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.printRepositories = exports.default = undefined;

var printRepositories = exports.printRepositories = function () {
  var _ref2 = _asyncToGenerator(regeneratorRuntime.mark(function _callee2() {
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            _context2.next = 2;
            return Promise.all(list.map(function (r) {
              return r.ready;
            }));

          case 2:
            list.forEach(printFiles);

          case 3:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function printRepositories() {
    return _ref2.apply(this, arguments);
  };
}();

exports.getRepo = getRepo;
exports.updateRepositories = updateRepositories;

var _terminalTableOutput = require('terminal-table-output');

var _terminalTableOutput2 = _interopRequireDefault(_terminalTableOutput);

var _pad = require('pad');

var _pad2 = _interopRequireDefault(_pad);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _socket = require('./socket');

var _repository = require('./repository');

var _repository2 = _interopRequireDefault(_repository);

var _log = require('./log');

var log = _interopRequireWildcard(_log);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var list = [];
exports.default = list;
function getRepo(assetPath) {
  var repoName = assetPath.match(/[/](.+?)[./]/)[1];
  return list.find(function (r) {
    return r.name === repoName;
  });
}

function updateRepositories() {
  var _this = this;

  var names = Object.keys(_config2.default.repositories);

  exports.default = list = list.reduce(function (list, r) {

    var isRemoved = !names.find(function (n) {
      return r.name === n && r.dir === _config2.default.repositories[n].path;
    });

    // this close() is verrrrryyyy slow, like sometimes more than 10 seconds.
    // so we just delay in hopes that it doesnt interrupt
    if (isRemoved) setTimeout(function () {
      return r.close();
    }, 1000);else list.push(r);

    return list;
  }, []);

  names.forEach(function (name) {
    if (list.find(function (r) {
      return r.name === name;
    })) return;

    var directory = _config2.default.repositories[name].path;
    var repo = new _repository2.default({ name: name, directory: directory });

    repo.on('message', function (e) {
      return (0, _socket.broadcast)(e);
    });

    setTimeout(_asyncToGenerator(regeneratorRuntime.mark(function _callee() {
      return regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              _context.next = 2;
              return repo.ready;

            case 2:
              printFiles(repo);

            case 3:
            case 'end':
              return _context.stop();
          }
        }
      }, _callee, _this);
    })));

    list.push(repo);
  });
}

function printFiles(repo) {
  log.info('\nRepository: ', repo.name);
  log.info('Directory: ', repo.dir);

  var table = _terminalTableOutput2.default.create().line().pushrow(['| Filename', 'Path']).line();

  var cols = process.stdout.columns / 2 - 3;

  repo.toJSON().files.forEach(function (f) {
    table.pushrow([('| ' + (0, _pad2.default)(f.filename, cols - 2)).slice(0, cols), (0, _pad2.default)(f.path || '', cols).slice(0, cols)]);
  });

  log.info(table.line().print());
}