'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.sendAsset = undefined;

var sendAsset = exports.sendAsset = function () {
  var _ref = _asyncToGenerator(regeneratorRuntime.mark(function _callee(req, res, next) {
    var repo, filename, useModules, requestText, ext;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (!(req.path === '/dev-server.js')) {
              _context.next = 2;
              break;
            }

            return _context.abrupt('return', res.sendFile(_path2.default.join(__dirname, '../client.js')));

          case 2:
            repo = (0, _repository_collection.getRepo)(req.path);

            // required for using this to serve assets for Codex server in development

            if (process.env.NODE_ENV === 'development') {
              res.setHeader('Access-Control-Allow-Methods', 'GET');
              res.setHeader('Access-Control-Allow-Origin', '*');
            }

            if (repo) {
              _context.next = 6;
              break;
            }

            return _context.abrupt('return', proxyAsset(req, res));

          case 6:
            filename = repo.findFilename(req.path);

            if (filename) {
              _context.next = 10;
              break;
            }

            log.error('Not found ' + req.path);
            return _context.abrupt('return', res.sendStatus(404));

          case 10:

            log.info('serving: /' + repo.name + '/' + filename);

            useModules = req.get('referer') && 'modules' in _url2.default.parse(req.get('referer'), true).query;
            requestText = req.get('Accept') && req.get('Accept').indexOf('text/plain') >= 0;
            ext = _path2.default.extname(req.path).slice(1);
            _context.prev = 14;

            if (!requestText) {
              _context.next = 20;
              break;
            }

            res.setHeader('Content-Type', 'text/plain; charset=utf-8');
            res.sendFile(_path2.default.join(repo.dir, filename));
            _context.next = 39;
            break;

          case 20:
            if (!(ext === 'css')) {
              _context.next = 29;
              break;
            }

            res.setHeader('Content-Type', 'text/css');
            _context.t0 = res;
            _context.next = 25;
            return repo.code(req.path);

          case 25:
            _context.t1 = _context.sent;

            _context.t0.send.call(_context.t0, _context.t1);

            _context.next = 39;
            break;

          case 29:
            if (!(ext === 'js')) {
              _context.next = 38;
              break;
            }

            res.setHeader('Content-Type', 'application/javascript');
            _context.t2 = res;
            _context.next = 34;
            return repo.code(req.path, { useModules: useModules });

          case 34:
            _context.t3 = _context.sent;

            _context.t2.send.call(_context.t2, _context.t3);

            _context.next = 39;
            break;

          case 38:
            res.sendFile(_path2.default.join(repo.dir, filename));

          case 39:
            _context.next = 44;
            break;

          case 41:
            _context.prev = 41;
            _context.t4 = _context['catch'](14);

            res.status(500).send(_context.t4.message);

          case 44:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this, [[14, 41]]);
  }));

  return function sendAsset(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();

exports.proxyAsset = proxyAsset;
exports.sendInline = sendInline;
exports.sendHTML = sendHTML;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _https = require('https');

var _https2 = _interopRequireDefault(_https);

var _uaParserJs = require('ua-parser-js');

var _uaParserJs2 = _interopRequireDefault(_uaParserJs);

var _url = require('url');

var _url2 = _interopRequireDefault(_url);

var _nodeCache = require('node-cache');

var _nodeCache2 = _interopRequireDefault(_nodeCache);

var _log = require('./log');

var log = _interopRequireWildcard(_log);

var _env = require('./env');

var env = _interopRequireWildcard(_env);

var _config = require('./config');

var _config2 = _interopRequireDefault(_config);

var _app = require('./app');

var _repository_collection = require('./repository_collection');

var _socket = require('./socket');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var cache = new _nodeCache2.default({ stdTTL: 600 });

function proxyAsset(req, res) {

  var cached = cache.get(req.path);

  if (cached) {
    // console.log('proxy cached: ' + req.path);
    return res.set(cached.headers).send(cached.body);
  }

  // console.log('proxy: ' + req.path);

  var options = {
    host: _url2.default.parse(env.contentOrigin).host,
    path: req.path
  };

  if (req.get('referer')) options.headers = { 'Referer': req.get('referer') };

  var pipe = function pipe(codexResponse) {

    var headerNames = ['etag', 'content-length', 'content-type', 'sourcemap', 'connection'];

    var headers = Object.keys(codexResponse.headers).reduce(function (headers, h) {
      if (headerNames.includes(h)) headers[h] = codexResponse.headers[h];
      return headers;
    }, {});

    res.writeHeader(codexResponse.statusCode, headers);

    var body = '';
    codexResponse.on('data', function (chunk) {
      return body += chunk;
    });

    codexResponse.on('error', function (error) {
      res.status(500).send(error.message);
      log.error(error);
    });

    codexResponse.on('end', function () {
      if (codexResponse.statusCode === 200) cache.set(req.path, { body: body, headers: headers });
      res.write(body);
      res.end();
    });
  };

  var sendError = function sendError(error) {
    res.status(500).send('Cannot load asset "' + req.path + ': ' + error.message);
  };

  _https2.default.request(options, pipe).on('error', sendError).end();
}

function sendInline(req, res, next) {
  var name = req.path.slice(1);
  var repo = (0, _repository_collection.getRepo)(name);

  if (repo && repo.hasInline(name)) {
    var filename = repo.inlineFilename(name);
    log.cyan('serving: ' + filename);
    res.sendFile(filename);
  } else next();
}

function sendHTML(req, res) {
  log.yellow('serving: ' + req.path);

  var environment = 'production';
  var contentOrigin = env.contentOrigin;
  var codexOrigin = env.codexOrigin;
  var assetOrigin = req.protocol + '://' + req.hostname + ':' + _app.port;
  var actionCableOrigin = 'wss://' + env.codexHost;

  var script = '/dev-server.js';
  var webpack = '';
  var webpackWS = '';

  if (process.env.NODE_ENV === 'development') {
    environment = 'development';
    actionCableOrigin = 'ws://' + env.codexHost + ' wss://' + env.codexHost;
    script = 'http://localhost:8001/main.js';
    webpack = 'http://localhost:8001/';
    webpackWS = 'ws://localhost:8001/';
  }

  var csp = ('\n    default-src  \'none\';\n    connect-src  \'self\' ' + webpack + ' ' + webpackWS + ' ' + codexOrigin + ' \n                 ' + contentOrigin + ' ws://' + req.get('Host') + '\n                 ' + actionCableOrigin + ' https://performance.typekit.net;\n    script-src   \'unsafe-eval\' \'self\' ' + webpack + '\n                 ' + codexOrigin + ' ' + contentOrigin + ' https://use.typekit.net\n                 https://connect.facebook.net;\n    style-src    \'unsafe-inline\' \'self\' ' + codexOrigin + ' ' + contentOrigin + '\n                 https://use.typekit.net https://fonts.googleapis.com;\n    font-src     data: \'self\' ' + contentOrigin + ' \n                 https://use.typekit.net https://fonts.gstatic.com;\n    img-src      blob: data: \'self\' ' + contentOrigin + ' https://p.typekit.net\n                 https://www.facebook.com;\n    media-src    blob: ' + contentOrigin + ';\n  ').replace(/\s+/g, ' ');

  if (!_config2.default.disableCSP) res.setHeader('Content-Security-Policy', csp);

  var data = {
    script: script, contentOrigin: contentOrigin, codexOrigin: codexOrigin, assetOrigin: assetOrigin, environment: environment
  };
  res.render('index.pug', data);
}

function browserAcceptsModules(userAgentString) {
  var ua = (0, _uaParserJs2.default)(userAgentString);
  var toInt = function toInt(str) {
    return typeof str == 'string' ? str.split(".")[0] : 0;
  };

  return ua.browser.name == 'Safari' && toInt(ua.browser.version) >= 10 || ua.browser.name == 'Mobile Safari' && toInt(ua.browser.version) >= 10 //||
  ;
}